package at.ac.uibk.gitsearch.web.rest;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.List;
import java.util.Optional;

import org.codeability.sharing.plugins.api.search.SearchResultsDTO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.InputStreamResource;
import org.springframework.core.io.Resource;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import at.ac.uibk.gitsearch.es.model.DocumentInfo;
import at.ac.uibk.gitsearch.service.SearchService;
import at.ac.uibk.gitsearch.service.StatisticsService;
import at.ac.uibk.gitsearch.service.dto.AutoCompleteEntry;
import at.ac.uibk.gitsearch.service.dto.StatisticsDTO;
import at.ac.uibk.gitsearch.web.util.HeaderUtil;

/**
 * REST controller for managing {@link DocumentInfo}.
 */
@RestController
@RequestMapping("/api")
@Transactional
public class SearchResource {

//  @Value("${jhipster.clientApp.name}") // warning: inconsistent rename of gitSearchApp to gitSearchV2App :-(
	private final static String applicationName = "gitsearchApp";

    private static final String ENTITY_NAME = "SearchResource";


    private final Logger log = LoggerFactory.getLogger(SearchResource.class);

    private final SearchService searchService;

    private final StatisticsService statisticsService;

    public SearchResource(SearchService searchService, StatisticsService statisticsService) {
        this.searchService = searchService;
        this.statisticsService = statisticsService;
    }

    /**
     * {@code SEARCH  /search/page-details} : search for the searchResults corresponding
     * to the query.
     *
     * @param query the query of the searchResult search.
     * @return the result of the search.
     */
    @PostMapping("/search/page-details")
    public SearchResultsDTO
    searchPageDetails(@RequestBody org.codeability.sharing.plugins.api.search.SearchInputDTO query) throws IOException {
        log.debug("REST request to search  {}", query);
        return searchService.searchResultPage(query);
    }

    /**
     * returns all keyword autocompletes for keyWord
     *
     * @param keyWordPrefix
     * @param max the maximum number of hits returned
     * @return
     * @throws IOException
     */
    @GetMapping("/search/keywordsAutoComplete")
    public List<AutoCompleteEntry> getKeywordsAutoComplete(@RequestParam String keyWordPrefix, @RequestParam(defaultValue = "10") int max) throws IOException {
        return searchService.getKeywordsAutoComplete(keyWordPrefix, max);
    }

        /**
     * returns all keyword autocompletes for keyWord
     *
     * @param formatPrefix
     * @param max the maximum number of hits returned
     * @return
     * @throws IOException
     */
    @GetMapping("/search/formatsAutoComplete")
    public List<AutoCompleteEntry> getFormatsAutoComplete(@RequestParam String formatPrefix, @RequestParam(defaultValue = "10") int max) throws IOException {
        return searchService.getFormatsAutoComplete(formatPrefix, max);
    }

    /**
     * returns all creator autocompletes
     *
     * @param creatorPrefix
     * @param max the maximum number of hits returned
     * @return
     * @throws IOException
     */
    @GetMapping("/search/creatorAutoComplete")
    public List<AutoCompleteEntry> getCreatorAutoComplete(@RequestParam String creatorPrefix, @RequestParam(defaultValue = "10") int max) throws IOException {
        return searchService.getCreatorAutoComplete(creatorPrefix, max);
    }

    /**
     * returns all contributor autocompletes
     *
     * @param contributorPrefix
     * @param max the maximum number of hits returned
     * @return
     * @throws IOException
     */
    @GetMapping("/search/contributorAutoComplete")
    public List<AutoCompleteEntry> getContributorAutoComplete(@RequestParam String contributorPrefix, @RequestParam(defaultValue = "10") int max) throws IOException {
        return searchService.getContributorAutoComplete(contributorPrefix, max);
    }

	/**
	 * returns all contributor and author autocompletes
	 *
	 * @param contributorPrefix
     * @param max the maximum number of hits returned
	 * @return
	 * @throws IOException
	 */
    @GetMapping("/search/contributorCreatorAutoComplete")
	public List<AutoCompleteEntry> getContributorCreatorAutoComplete(String contributorPrefix, @RequestParam(defaultValue = "10") int max) throws IOException {
		return searchService.getContributorCreatorAutoComplete(contributorPrefix, max);
	}

    /**
     * returns all programmingLanguage autocompletes for keyWord
     *
     * @param programmingLanguagePrefix
     * @param max the maximum number of hits returned
     * @return
     * @throws IOException
     */
    @GetMapping("/search/programmingLanguageAutoComplete")
    public List<AutoCompleteEntry> getProgrammingLanguageAutoComplete(@RequestParam String programmingLanguagePrefix, @RequestParam(defaultValue = "10") int max) throws IOException {
        return searchService.getProgrammingLanguageAutoComplete(programmingLanguagePrefix, max);
    }


    /**
     * POST /programming-exercises/:exerciseId/export-programmingExercise : sends all repositories and details of the programming exercise as zip
     * TODO projectId and ExerciseId are mixed up here. Please clarify!
     * 
     * @param projectId the id of the exercise to get the repos from
     * ResponseEntity with status
     * @throws IOException if something during the zip process went wrong
     */
    @PostMapping("/programming-exercises/{projectId}/export-programming-exercise")
    public ResponseEntity<Resource> exportProgrammingExercise(@PathVariable long projectId) throws IOException {

        File zipFile = searchService.exportExercise(projectId);
        
        if (zipFile == null) {
            return ResponseEntity.badRequest().headers(HeaderUtil.createFailureAlert(applicationName, true, ENTITY_NAME, "internalServerError",
                    "There was an error on the server and the zip file could not be created.")).body(null);
        }

        /*
         * Customized FileInputStream to delete and therefore clean up the returned files
         */
        class NewFileInputStream extends FileInputStream {

            File file;

            public NewFileInputStream(File file) throws FileNotFoundException {
                super(file);
                this.file = file;
            }

            public void close() throws IOException {
                super.close();
                if(!file.delete()) {
                	log.info("Cannot delete {}", file.getAbsoluteFile());
                }
            }

        }
        InputStreamResource resource = new InputStreamResource(new NewFileInputStream(zipFile));

        log.debug("REST request to get Statistics for ExerciseID : {}", projectId);
        Optional<StatisticsDTO> statisticsDTO = statisticsService.findOneByExerciseID(projectId);
        if(statisticsDTO.isPresent()){
            StatisticsDTO newStats = statisticsDTO.get();
            newStats.setDownloads(newStats.getDownloads() + 1);
            statisticsService.save(newStats);
            log.debug("REST increased number of downloads for ExerciseID : {}", projectId);
        }
        if(!statisticsDTO.isPresent()){
            StatisticsDTO newStats = new StatisticsDTO();
            newStats.setDownloads(1);
            newStats.setViews(1);
            newStats.setExerciseID(projectId);
            statisticsService.save(newStats);
            log.debug("Created new statistics entry for exerciseID: {}", projectId);
        }

        return ResponseEntity.ok().contentLength(zipFile.length()).contentType(MediaType.APPLICATION_OCTET_STREAM).header("filename", zipFile.getName()).body(resource);
    }


}
